@@ -7,7 +7,7 @@
 #' @param TR matrix or data.frame of trait covariates.
 #' @param data data in long format, that is, matrix of responses, environmental and trait covariates and row index named as "id". When used, model needs to be defined using formula. This is alternative data input for y, X and TR.
 #' @param formula an object of class "formula" (or one that can be coerced to that class): a symbolic description of the model to be fitted (for fixed-effects predictors).
-#' @param family  distribution function for responses. Options are \code{"negative.binomial"} (with log link), \code{poisson(link = "log")}, \code{binomial(link = "probit")} (and also with \code{link = "logit"} when \code{method = "LA"} or \code{method = "EVA"}), zero inflated poisson (\code{"ZIP"}), \code{gaussian(link = "identity")}, Tweedie (\code{"tweedie"}) (with log link, for \code{"LA"} and \code{"EVA"}-method), \code{"gamma"} (with log link), \code{"exponential"} (with log link), beta (\code{"beta"}) (with logit and probit link, for \code{"LA"} and  \code{"EVA"}-method) and \code{"ordinal"} (only with \code{"VA"}-method).
+#' @param family  distribution function for responses. Options are \code{"negative.binomial"} (with log link), \code{poisson(link = "log")}, \code{binomial(link = "probit")} (and also with \code{link = "logit"} when \code{method = "LA"} or \code{method = "EVA"}), zero-inflated poisson (\code{"ZIP"}), zero-inflated negative-binomial (\code{"ZINB"}), \code{gaussian(link = "identity")}, Tweedie (\code{"tweedie"}) (with log link, for \code{"LA"} and \code{"EVA"}-method), \code{"gamma"} (with log link), \code{"exponential"} (with log link), beta (\code{"beta"}) (with logit and probit link, for \code{"LA"} and  \code{"EVA"}-method) and \code{"ordinal"} (only with \code{"VA"}-method).
 #' @param num.lv  number of latent variables, d, in gllvm model. Non-negative integer, less than number of response variables (m). Defaults to 2, if \code{num.lv.c=0} and \code{num.RR=0}, otherwise 0.
 #' @param num.lv.c  number of latent variables, d, in gllvm model to constrain, with residual term. Non-negative integer, less than number of response (m) and equal to, or less than, the number of predictor variables (k). Defaults to 0. Requires specification of "lv.formula" in combination with "X" or "datayx". Can be used in combination with num.lv and fixed-effects, but not with traits.
 #' @param num.RR number of latent variables, d, in gllvm model to constrain, without residual term (reduced rank regression). Cannot yet be combined with traits.
@@ -22,8 +22,9 @@
 #' @param randomB either \code{FALSE}(default), "LV", "P", or "single". Fits concurrent or constrained ordination (i.e. models with num.lv.c or num.RR) with random slopes for the predictors. "LV" assumes LV-specific variance parameters, "P" predictor specific, and "single" the same across LVs and predictors.
 #' @param sd.errors  logical. If \code{TRUE} (default) standard errors for parameter estimates are calculated.
 #' @param offset vector or matrix of offset terms.
+#' @param Ntrials number of trials for binomial family.
 #' @param link link function for binomial family if \code{method = "LA"} and beta family. Options are "logit" and "probit.
-#' @param Power fixed power parameter in Tweedie model. Scalar from interval (1,2). Defaults to 1.1.
+#' @param Power fixed power parameter in Tweedie model. Scalar from interval (1,2). Defaults to 1.1. If set to NULL it is estimated (note: experimental). 
 #' @param seed a single seed value, defaults to \code{NULL}.
 #' @param plot  logical. If \code{TRUE} ordination plots will be printed in each iteration step when \code{TMB = FALSE}. Defaults to \code{FALSE}.
 #' @param zeta.struc structure for cut-offs in the ordinal model. Either "common", for the same cut-offs for all species, or "species" for species-specific cut-offs. For the latter, classes are arbitrary per species, each category per species needs to have at least one observations. Defaults to "species".
@@ -50,6 +51,7 @@
 #' \itemize{
 #'  \item{\emph{Lambda.struc}: }{ covariance structure of VA distributions for latent variables when \code{method = "VA"}, "unstructured" or "diagonal".}
 #'  \item{\emph{Ab.struct}: }{ covariance structure of VA distributions for random slopes when \code{method = "VA"}, "unstructured" or "diagonal".}
+#'  \item{\emph{Ar.struc}: }{ covariance structure of VA distributions for random row effects when \code{method = "VA"}, "unstructured" or "diagonal".}
 #'  \item{\emph{diag.iter}: }{ non-negative integer which can sometimes be used to speed up the updating of variational (covariance) parameters in VA method. Can sometimes improve the accuracy. If \code{TMB = TRUE} either 0 or 1. Defaults to 1.}
 #'  \item{\emph{Ab.diag.iter}: }{ As above, but for variational covariance of random slopes.}
 #'  \item{\emph{Lambda.start}: }{ starting values for variances in VA distributions for latent variables, random row effects and random slopes in variational approximation method. Defaults to 0.3.}
@@ -67,7 +69,8 @@
 #'   \item{\emph{start.struc}: }{ starting value method for the quadratic term. Options are \code{"LV"} (default) and \code{"all"}.}
 #'   \item{\emph{quad.start}: }{ starting values for quadratic coefficients. Defaults to 0.01.}
 #'   \item{\emph{MaternKappa}: }{ Starting value for smoothness parameter kappa of Matern covariance function. Defaults to 3/2.}
-#'   \item{\emph{scalmax}: }{ Sets starting value for the range/scale parameter for the coordinates. Defaults to 10, when the starting value for scale parameter scales the coordinates between 0-10.}
+#'   \item{\emph{scalmax}: }{ Sets starting value for the scale parameter for the coordinates. Defaults to 10, when the starting value for scale parameter scales the distances of coordinates between 0-10.}
+#'   \item{\emph{rangeP}: }{ Sets starting value for the range parameter for the correlation structure.}
 #' }
 #' @param ... Not used.
 #'
@@ -95,9 +98,9 @@
 #'}
 #'
 #' \subsection{Ordination with predictors}{
-#'For GLLVMs with both linear and quadratic response model, a series of predictors \eqn{x_lv} can be included to explain the latent variables:
+#'For GLLVMs with both linear and quadratic response model, a series of predictors \eqn{x_{lv}} can be included to explain the latent variables:
 #'
-#'\deqn{g(\mu_{ij}) = \alpha_i + \beta_{0j} + x_i'\beta_j + (B' x_{lv,i} + \epsilon_i)' \gamma_j - (B' x_{lv,i})' D_j (B' x_{lv,i} + \epsilon_i) ,}
+#'\deqn{g(\mu_{ij}) = \alpha_i + \beta_{0j} + x_i'\beta_j + (B' x_{lv,i} + \epsilon_i)' \gamma_j - (B' x_{lv,i} + \epsilon_i)' D_j (B' x_{lv,i} + \epsilon_i) ,}
 #'where \eqn{z_i = B' x_{lv,i} + \epsilon_i} are latent variables informed by the predictors, but not constrained compared to unconstrained ordination as in methods such as CCA or RDA.
 #' Omitting the predictors results in an unconstrained ordination, and omitting \eqn{\epsilon_i} in the usual constrained ordination, which can also be fitted.
 #'}
@@ -149,8 +152,8 @@
 #'\itemize{
 #'   \item{For count data \code{family = poisson()}:} {Expectation \eqn{E[Y_{ij}] = \mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}}, or}
 #'   \item{ \code{family = "negative.binomial"}:}{ Expectation \eqn{E[Y_{ij}] = \mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}+\mu_{ij}^2\phi_j}, or}
-#'   \item{ \code{family = "ZIP"}:}{ Expectation \eqn{E[Y_{ij}] = (1-p)\mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}(1-p)(1+\mu_{ij}p)}.}
-#'
+#'   \item{ \code{family = "ZIP"}:}{ Expectation \eqn{E[Y_{ij}] = (1-p)\mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}(1-p_j)(1+\mu_{ij}p)}.}
+#'   \item{ \code{family = "ZINB"}:}{ Expectation \eqn{E[Y_{ij}] = (1-p)\mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}(1-p_j)(1+\mu_{ij}(\phi_j+p_j))}.}
 #'   \item{For binary data \code{family = binomial()}:}{ Expectation \eqn{E[Y_{ij}] = \mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}(1-\mu_{ij})}.}
 #'   
 #'   \item{For percent cover data \eqn{0 < Y_{ij} < 1} \code{family = "beta"}:}{ Expectation \eqn{E[Y_{ij}] = \mu_{ij}}, variance \eqn{V(\mu_{ij}) = \mu_{ij}(1-\mu_{ij})//(1+\phi_j)}.}
@@ -184,6 +187,7 @@
 #'  \item{num.lv}{ Number of unconstrained latent variables}
 #'  \item{num.lv.c}{ Number of latent variables in concurrent ordination}
 #'  \item{num.RR}{ Number of latent variables in constrained ordination}
+#'  \item{Ntrials}{ Number of trials in a binomial model}
 #'  \item{method}{ Method used for integration}
 #'  \item{family}{ Response distribution}
 #'  \item{row.eff}{ Type of row effect used}
@@ -233,7 +237,7 @@
 #'
 #' Kasper Kristensen, Anders Nielsen, Casper W. Berg, Hans Skaug, Bradley M. Bell (2016). TMB: Automatic Differentiation and Laplace Approximation. Journal of Statistical Software, 70(5), 1-21.
 #'
-#' Korhonen, P., Hui, F. K. C., Niku, J., and Taskinen, S. (2021). Fast, universal estimation of latent variable models using extended variational approximations. Submitted.
+#' Korhonen, P., Hui, F. K. C., Niku, J., and Taskinen, S. (2021). Fast, universal estimation of latent variable models using extended variational approximations. Stat Comput 33, 26 (2023).
 #'
 #' Niku, J., Warton,  D. I., Hui, F. K. C., and Taskinen, S. (2017). Generalized linear latent variable models for multivariate count and biomass data in ecology. Journal of Agricultural, Biological, and Environmental Statistics, 22:498-522.
 #'
@@ -286,6 +290,11 @@
 #'               num.lv.c=2, method="VA")
 #'ordiplot(fity1, biplot = TRUE)
 #'
+#'#'## Example 1b: Fit constrained ordination model with two latent variables and with 
+#'# random canonical coefficients
+#'fity2 <- gllvm(y, X = scale(X), family = "negative.binomial", 
+#'               num.RR=2, randomB="LV", method="VA")
+#'               
 #'# Using Laplace approximation: (this line may take about 30 sec to run)
 #'fitl0 <- gllvm(y, family = "negative.binomial", method = "LA")
 #'ordiplot(fitl0)
@@ -383,7 +392,8 @@
 #'@importFrom mgcv gam predict.gam
 #'@importFrom nloptr nloptr
 #'@importFrom alabama auglag
-#'@importFrom utils combn tail
+#'@importFrom utils combn tail relist
+#'@importFrom methods cbind2 rbind2
 #'
 
 
@@ -392,18 +402,21 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
                   num.lv = NULL, num.lv.c = 0, num.RR = 0, lv.formula = NULL,
                   lvCor = NULL, studyDesign=NULL,dist = matrix(0), corWithin = FALSE, quadratic = FALSE, 
                   row.eff = FALSE, sd.errors = TRUE, offset = NULL, method = "VA", randomB = FALSE,
-                  randomX = NULL, dependent.row = FALSE, beta0com = FALSE, zeta.struc="species",
-                  plot = FALSE, link = "probit", 
-                  Power = 1.1, seed = NULL, scale.X = TRUE, return.terms = TRUE, gradient.check = FALSE, disp.formula = NULL,
-                  control = list(reltol = 1e-10, reltol.c = 1e-8, TMB = TRUE, optimizer = ifelse((num.RR+num.lv.c)==0 | randomB!=FALSE,"optim","alabama"), max.iter = 4000, maxit = 4000, trace = FALSE, optim.method = NULL), 
-                  control.va = list(Lambda.struc = "unstructured", Ab.struct = "unstructured", Ar.struc="unstructured", diag.iter = 1, Ab.diag.iter=0, Lambda.start = c(0.3, 0.3, 0.3), NN = 3),
-                  control.start = list(starting.val = "res", n.init = 1, n.init.max = 10, jitter.var = 0, start.fit = NULL, start.lvs = NULL, randomX.start = "zero", quad.start=0.01, start.struc = "LV", scalmax = 10, MaternKappa=1.5), setMap=NULL, ...
+                  randomX = NULL, dependent.row = FALSE, beta0com = FALSE, zeta.struc = "species",
+                  plot = FALSE, link = "probit", Ntrials = 1,
+                  Power = 1.1, seed = NULL, scale.X = TRUE, return.terms = TRUE, 
+                  gradient.check = FALSE, disp.formula = NULL,
+                  control = list(reltol = 1e-10, reltol.c = 1e-8, TMB = TRUE, optimizer = ifelse((num.RR+num.lv.c)==0 | randomB!=FALSE,"optim","alabama"), max.iter = 6000, maxit = 6000, trace = FALSE, optim.method = NULL), 
+                  control.va = list(Lambda.struc = "unstructured", Ab.struct = "unstructured", Ar.struc="diagonal", diag.iter = 1, Ab.diag.iter=0, Lambda.start = c(0.3, 0.3, 0.3), NN = 3),
+                  control.start = list(starting.val = "res", n.init = 1, n.init.max = 10, jitter.var = 0, start.fit = NULL, start.lvs = NULL, randomX.start = "zero", quad.start=0.01, start.struc = "LV", scalmax = 10, MaternKappa=1.5, rangeP=NULL), setMap=NULL, ...
                   ) {
   # Dthreshold=0,
+  if(!method%in%c("LA","VA","EVA"))stop("Selected method is not supported.")
+  if(method=="EVA" && !isFALSE(quadratic))stop("The EVA method is not available for a quadratic GLLVM.")
   if(!is.null(lvCor)) warning("'lvCor' is under development, so all properties may not work properly.")
     #change default behavior of num.lv.
     #if num.lv.c>0, num.lv defaults to 0 if it is 0. Otherwise, it defaults to 2
-  if(randomB!=FALSE&quadratic!=FALSE&(num.lv.c+num.RR)>0&method=="LA"){
+  if(randomB!=FALSE&&quadratic!=FALSE&&(num.lv.c+num.RR)>0&&method=="LA"){
     stop("Model with quadratic responses and random slopes not allowed with method 'LA'")
   }
   if((num.RR+num.lv.c)==0){
@@ -440,27 +453,29 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
         x$TMB = TRUE
       if (!("optimizer" %in% names(x))) 
         x$optimizer = ifelse((num.RR+num.lv.c)<=1 | randomB!=FALSE,"optim","alabama")
-        if((num.lv.c+num.RR)>1 && family =="tweedie") x$optimizer = "alabama"
+      if((num.lv.c+num.RR)>1 && family =="tweedie") x$optimizer = "alabama"
       if (!("optim.method" %in% names(x)) | is.null(x$optim.method)) {
         if(family=="tweedie") x$optim.method = "L-BFGS-B" else x$optim.method = "BFGS"
         if((num.RR+num.lv.c)>1 && randomB == FALSE && family!="tweedie" && x$optimizer%in%c("nloptr(agl)","nloptr(sqp)")) x$optim.method = "NLOPT_LD_TNEWTON_PRECOND"
-        }
+      }
       if (!("max.iter" %in% names(x))) 
-        x$max.iter = 200
+        x$max.iter = 6000
       if (!("maxit" %in% names(x))) 
-        x$maxit = 4000
+        x$maxit = 6000
       if (!("trace" %in% names(x))) 
         x$trace = FALSE
       x
     }
     
     fill_control.va = function(x){
-      if (!("Lambda.struc" %in% names(x))) 
+      if (!("Lambda.struc" %in% names(x)) & is.null(lvCor)) 
         x$Lambda.struc = "unstructured"
+      if (!("Lambda.struc" %in% names(x)) & !is.null(lvCor)) 
+        x$Lambda.struc = "diagonal"
       if (!("Ab.struct" %in% names(x))) 
         x$Ab.struct = "unstructured"
       if (!("Ar.struc" %in% names(x))) 
-        x$Ar.struc = "unstructured"
+        x$Ar.struc = "diagonal"
       if (!("diag.iter" %in% names(x))) 
         x$diag.iter = 5
       if (!("Ab.diag.iter" %in% names(x))) 
@@ -492,6 +507,8 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
         x$start.struc = "LV"
       if (!("scalmax" %in% names(x))) 
         x$scalmax = 10
+      if (!("rangeP" %in% names(x))) 
+        x$rangeP = NULL
       if (!("MaternKappa" %in% names(x))) 
         x$MaternKappa = 1.5
       x
@@ -507,7 +524,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     control.start <- fill_control.start(c(pp.pars, control.start))
     
   #some checks for optimizer
-    
+
   # Cannot use nloptr or alabama with randomB
   if(randomB!=FALSE && control$optimizer %in% c("alabama","nloptr(sqp)","nloptr(agl)")){
     warning("Random slope models should use 'nlminb' or 'optim' as optimizer. Changing to 'optim'.")
@@ -552,14 +569,19 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     if(randomB!=FALSE&!control$TMB){
       stop("Random slopes in ordination only allows with TMB = TRUE.")
     }
-    
+    if(family == "binomial" && max(Ntrials) == 1 && !is.null(y) && max(y, na.rm=TRUE)>1){
+    stop("Using the binomial distribution requires setting the `Ntrials` argument.")
+    }
+    if(family == "binomial" && method == "EVA" && max(Ntrials) >1){
+      stop("Binomial distribution not yet supported with the EVA method.")
+    }
     # if(num.RR>0&quadratic>0&(num.lv+num.lv.c)==0){
     #   control.start$start.struc <- "all"
     # }
     reltol = control$reltol; reltol.c = control$reltol.c; TMB = control$TMB; optimizer = control$optimizer; max.iter = control$max.iter; maxit = control$maxit; trace = control$trace; optim.method = control$optim.method
     Lambda.struc = control.va$Lambda.struc; Ab.struct = control.va$Ab.struct; Ar.struc = control.va$Ar.struc; diag.iter = control.va$diag.iter; Ab.diag.iter=control.va$Ab.diag.iter; Lambda.start = control.va$Lambda.start; NN = control.va$NN;
     starting.val = control.start$starting.val; n.init = control.start$n.init; n.init.max = control.start$n.init.max; jitter.var = control.start$jitter.var; start.fit = control.start$start.fit; start.lvs = control.start$start.lvs; randomX.start = control.start$randomX.start
-    start.struc = control.start$start.struc;quad.start=control.start$quad.start;scalmax=control.start$scalmax; MaternKappa=control.start$MaternKappa
+    start.struc = control.start$start.struc;quad.start=control.start$quad.start; scalmax=control.start$scalmax; rangeP=control.start$rangeP; MaternKappa=control.start$MaternKappa
     
     
     if(!is.null(TR)&num.lv.c>0|!is.null(TR)&num.RR>0){
@@ -590,10 +612,14 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     if(!is.null(X)){
       if(!is.matrix(X) && !is.data.frame(X) ) 
         stop("X must be a matrix or data.frame.")
+      if(any(is.na(X)) )
+        stop("NAs are not allowed in 'X'.")
     }
     if(!is.null(TR)){
-      if(!is.matrix(TR) && !is.data.frame(TR) ) 
+      if(!is.matrix(TR) && !is.data.frame(TR) )
         stop("TR must be a matrix or data.frame.")
+      if(any(is.na(TR)) )
+        stop("NAs are not allowed in 'TR'.")
     }
     #is.null(X)&is.null(data)&num.lv.c>0|
     if((num.RR+num.lv.c)>0&is.null(X)&is.null(data)){
@@ -621,6 +647,10 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       }
       }
     }
+    if((num.lv.c+num.RR)>0&!is.null(formula)&is.null(lv.formula)){
+      stop("'lv.formula' should be provided when 'formula' is used with concurrent or constrained ordination.")
+    }
+    
     
     if (!is.null(y)) {
       y <- as.matrix(y)
@@ -641,7 +671,12 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
         } else if(is.null(formula)&is.null(lv.formula)&(num.lv.c+num.RR)>0){
 
           if(inherits(row.eff,"formula")){
-            lv.formula <- formula(paste("~", paste("+", colnames(X[,-which(colnames(X)==all.vars(row.eff))]), collapse = "")))
+            if(any(colnames(X)==all.vars(row.eff))){
+              lv.formula <- formula(paste("~", paste(colnames(X[,-which(colnames(X)==all.vars(row.eff))]), collapse = "+")))              
+            }else{
+              lv.formula <- formula(paste("~", paste(colnames(X), collapse = "+")))
+            }
+
             if (is.data.frame(X)) {
               datayx <- list(X = model.matrix(lv.formula, X)[,-1,drop=F])
             } else {
@@ -649,7 +684,11 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
             }
             lv.X <- as.matrix(model.frame(~ X, data = datayx))
             if(!is.null(row.names(lv.X)))row.names(lv.X)<-row.names(X)
-            X <-  X[,all.vars(row.eff),drop=F]
+            if(any(colnames(X)==all.vars(row.eff))){
+              X <-  X[,all.vars(row.eff),drop=F]
+            }else{
+             X <- NULL 
+            }
           }else{
             lv.formula <- formula(paste("~", paste(colnames(X), collapse = "+")))
             if (is.data.frame(X)) {
@@ -671,7 +710,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
           m1 <- model.frame(formula, data = datayx)
           term <- terms(m1)
           lv.X <- NULL
-          lv.formula <- y ~ NULL
+          lv.formula <- ~ 1
         } else if(is.null(formula)&!is.null(lv.formula)){
           if(inherits(row.eff,"formula")){
             if(any(colnames(X)==all.vars(row.eff))){
@@ -679,6 +718,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
               X <-  X[,all.vars(row.eff),drop=F]
             } else {
               datayx <- data.frame(y, X)
+              X <- NULL
             }
           }else{
             datayx <- data.frame(y, X)
@@ -692,7 +732,6 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
 
           lv.formula <- formula(paste("~", paste(labterm, collapse = "+")))
           lv.X<- model.matrix(lv.formula,data=datayx)[,-1,drop=F]
-     
         }else if(!is.null(formula)&!is.null(lv.formula)){
           datayx <- data.frame(y, X)
           m1 <- model.frame(formula, data = datayx)
@@ -861,7 +900,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       y <- as.matrix(y)
 
 # Structured row parameters
-    rstruc = 0; dr = NULL; cstruc = "diag"
+    rstruc = 0; dr = NULL; dLV = NULL; cstruc = c("diag","diag")
     if(inherits(row.eff,"formula")) {
       bar.f <- findbars1(row.eff) # list with 3 terms
       grps <- unlist(lapply(bar.f,function(x) as.character(x[[3]])))
@@ -871,7 +910,8 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
           xgrps<-as.data.frame(data[1:n,(colnames(data) %in% grps)])
           colnames(xgrps) <- grps
           studyDesign<-cbind(studyDesign, xgrps)
-          }
+        }
+          
       } else if(all(grps %in% colnames(X))) {
         if (!is.null(studyDesign)){ 
           studyDesign=cbind(studyDesign, X)
@@ -885,6 +925,9 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       } else if(is.null(studyDesign)){
         stop("Grouping variable needs to be included in 'studyDesign'")
       }
+      } else if(!is.null(studyDesign) && any(colnames(studyDesign) %in% colnames(X))){
+        X <- X[,-which(colnames(X)%in%colnames(studyDesign)),drop=F]
+        if(ncol(X)==0)X<-NULL
       }
       
       if(is.null(bar.f)) {
@@ -894,13 +937,12 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       } else if(!all(order(studyDesign[,(colnames(studyDesign) %in% grps)])==c(1:n)) && (corWithin)) {
         stop("Data (response matrix Y and covariates X) needs to be grouped according the grouping variable: '",grps,"'")
       } else {
-        if(quadratic != FALSE) {warning("Structured row effects model may not work properly with the quadratic model yet.")}
         mf <- model.frame(subbars1(row.eff),data=studyDesign)
         dr <- t(as.matrix(mkReTrms1(bar.f,mf)$Zt))
         if(corWithin){ rstruc=2} else { rstruc=1}
       }
-      cstruc = corstruc(row.eff)[1]
-      if(cstruc == "diag" & rstruc==2) {rstruc=0; dr=NULL}
+      cstruc[1] = corstruc(row.eff)[1]
+      if(cstruc[1] == "diag" & rstruc==2) {rstruc=0; dr=NULL}
       row.eff = "random"
     }
     
@@ -933,11 +975,11 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       } else if(!all(order(studyDesign[,(colnames(studyDesign) %in% grps)])==c(1:n)) && (corWithin)) {
         stop("Data (response matrix Y and covariates X) needs to be grouped according the grouping variable: '",grps,"'")
       } else {
-        if(quadratic != FALSE) {warning("Structured row effects model may not work properly with the quadratic model yet.")}
+        # if(quadratic != FALSE) {warning("Structured row effects model may not work properly with the quadratic model yet.")}
         mf <- model.frame(subbars1(lvCor),data=studyDesign)
-        dr <- t(as.matrix(mkReTrms1(bar.lv,mf)$Zt))
+        dLV <- t(as.matrix(mkReTrms1(bar.lv,mf)$Zt))
       }
-      cstruc = corstruc(lvCor)[1]
+      cstruc[2] = corstruc(lvCor)[1]
       num.lv.cor = num.lv
       # Have to set this to diagonal to avoid too many parameters:
       # Lambda.struc = "diagonal"
@@ -954,26 +996,29 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       # indM[,1]>indM[,2]
       indM<-indM[order(indM[,2]),]
       indM<-indM[order(indM[,1]),]
-      dupl<-c(TRUE, rowSums(abs(indM[-1,]-indM[1:(nrow(indM)-1),]))!=0)
+      dupl<-c(TRUE, rowSums(abs(indM[-1,]-indM[1:(nrow(indM)-1),]), na.rm=TRUE)!=0)
       NN<-indM[dupl,]
+    } else if(Lambda.struc %in% c("LR") & num.lv.cor>0){
+      NN <- as.matrix(NN)
     } else {NN=matrix(0)}
     
     if(num.lv==0&num.lv.c==0&num.RR==0)quadratic <- FALSE
 
-    if(any(colSums(y)==0))
+    if(any(colSums(y, na.rm = TRUE) == 0))
       warning("There are responses full of zeros. \n");
 
     if(row.eff %in% c("fixed", "random", TRUE) ){
       if((p<2) & (rstruc == 0))
         stop("There must be at least two responses in order to include unstructured row effects. \n");
-      if(any(rowSums(y)==0))
+      if(any(rowSums(y, na.rm = TRUE)==0))
         warning("There are rows full of zeros in y. \n");
       }
     # if(row.eff == "random" && quadratic != FALSE && Lambda.struc == "unstructured"){
     #   stop("Dependent row-effects can only be used with quadratic != FALSE if Lambda.struc == 'diagonal'' '. \n")
     #   #This can potentially be relaxed for the gaussian, binomial and ordinal distributions because the linear and quadratic approximation terms can be separated.
     # }
-
+    if( any(!is.finite(y[!is.na(y)])) ) stop("Infinite values are not allowed in 'y'")
+    if(any(is.na(y)))y[is.na(y)]<-NA_real_
     if (row.eff == "random" && family == "ordinal" && TMB==FALSE) {
       stop("Random row effect model is not implemented for ordinal family without TMB. \n")
     }
@@ -994,10 +1039,10 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       cat("Laplace's method is not implemented without TMB, so 'TMB = TRUE' is used instead. \n")
       TMB = TRUE
     }
-    if ((method %in% c("VA", "EVA")) && (family == "ZIP")) {
-      cat("VA method cannot handle", family, " family, so LA method is used instead. \n")
-      method <- "LA"
-    }
+    # if ((method %in% c("VA", "EVA")) && (family == "ZIP")) {
+    #   cat("VA method cannot handle", family, " family, so LA method is used instead. \n")
+    #   method <- "LA"
+    # }
     if (quadratic != FALSE && family %in% c("tweedie", "beta")){
       stop("The quadratic model is not implemented for ", family, " family yet. \n")
     }
@@ -1015,7 +1060,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       TMB <- TRUE
     }
     
-    if (family == "gaussian" && !TMB) {
+    if (family %in% c("gaussian","ZIP","ZINB","beta","Tweedie","gamma","exponential") && !TMB) {
       TMB <- TRUE
       cat("Only TMB implementation available for ", family, " family, so 'TMB = TRUE' is used instead. \n")
     }
@@ -1043,7 +1088,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     # where unconstrained LVs are not uncorrelated with predictors
     # better inform the user this might be a bad idea
     if(!is.null(lv.X)){
-      if((num.RR+num.lv.c)>0&(num.lv>0)&((num.RR+num.lv.c)<ncol(lv.X))){
+      if(num.lv.c>0&(num.lv>0)&((num.RR+num.lv.c)<ncol(lv.X))){
         warning("Are you sure you want to fit this model? It might be better to increase num.RR or num.lv.c until the number of predictors is reached, before adding unconstrained LVs. \n")
       }
     }
@@ -1071,7 +1116,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
 
     out <- list( y = y, X = X, lv.X = lv.X, TR = TR, data = datayx, num.lv = num.lv, num.lv.c = num.lv.c, num.RR = num.RR, num.lvcor =num.lv.cor, lv.formula = lv.formula, lvCor = lvCor, formula = formula,
         method = method, family = family, row.eff = row.eff, rstruc =rstruc, corP=list(cstruc = cstruc, corWithin=corWithin, Astruc=0), dist=dist, randomX = randomX, n.init = n.init,
-        sd = FALSE, Lambda.struc = Lambda.struc, TMB = TMB, beta0com = beta0com, optim.method=optim.method, disp.group = disp.group, NN=NN)
+        sd = FALSE, Lambda.struc = Lambda.struc, TMB = TMB, beta0com = beta0com, optim.method=optim.method, disp.group = disp.group, NN=NN, Ntrials = Ntrials, quadratic = quadratic, randomB = randomB)
     if(return.terms) {out$terms = term} #else {terms <- }
 
     if("la.link.bin" %in% names(pp.pars)){link = pp.pars$la.link.bin}
@@ -1081,12 +1126,31 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       if (method == "VA")
         out$link <- "probit"
     }
-    if (family == "beta") {
+    if (family %in% c("beta","betaH","orderedBeta")) {
         out$link <- link
     }
+
+    if (family == "orderedbeta") {
+      out$link <- "probit"
+    }
     out$offset <- offset
     if(quadratic=="LV")start.struc <- "LV"
     if (TMB) {
+      if (family == "betaH") {
+        if(is.null(colnames(y))) colnames(y)= paste("y",1:NCOL(y))
+        y01 = y #(y>0)*1; 
+        if(is.null(colnames(y01))) colnames(y01)= paste("y",1:NCOL(y01))
+        colnames(y01) = paste("H01",colnames(y01), sep = "_")
+        y=cbind(y,y01)
+        if(!is.null(TR)){
+          TR=rbind(TR,TR)
+        }
+        if(!is.null(disp.group)){
+          disp.group=c(disp.group,disp.group)
+        }
+        O = cbind(O,O)
+      }
+      
       # trace = FALSE
       if (row.eff == TRUE)
         row.eff <- "fixed"
@@ -1136,10 +1200,11 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
             zeta.struc = zeta.struc,
             quadratic = quadratic,
             optim.method=optim.method, 
-            dr=dr, rstruc =rstruc, cstruc = cstruc, dist =dist, corWithin = corWithin, NN=NN, 
-            scalmax = scalmax, MaternKappa = MaternKappa,
+            dr=dr,dLV=dLV, rstruc =rstruc, cstruc = cstruc, dist =dist, corWithin = corWithin, NN=NN, 
+            scalmax = scalmax, MaternKappa = MaternKappa, rangeP = rangeP,
             setMap = setMap, #Dthreshold=Dthreshold,
-            disp.group = disp.group
+            disp.group = disp.group,
+            Ntrials = Ntrials
         )
         out$X <- fitg$X
         out$TR <- fitg$TR
@@ -1172,6 +1237,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
             sd.errors = sd.errors,
             trace = trace,
             link = link,
+            Ntrials = Ntrials,
             n.init = n.init,
             n.init.max = n.init.max,
             restrict = restrict,
@@ -1187,8 +1253,8 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
             quadratic = quadratic,
             randomB = randomB,
             optim.method=optim.method, 
-            dr=dr, rstruc =rstruc, cstruc = cstruc, dist =dist, corWithin = corWithin, NN=NN, 
-            scalmax = scalmax, MaternKappa = MaternKappa,
+            dr=dr, dLV=dLV, rstruc =rstruc, cstruc = cstruc, dist =dist, corWithin = corWithin, NN=NN, 
+            scalmax = scalmax, MaternKappa = MaternKappa, rangeP = rangeP,
             setMap=setMap, #Dthreshold=Dthreshold,
             disp.group = disp.group
         )
@@ -1196,31 +1262,39 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
           out$formula <- fitg$formula
           out$X <- fitg$X
         }
-      }
+      
+}
       out$disp.group <- disp.group
       out$seed <- fitg$seed
       out$X.design <- fitg$X.design
       out$TMBfn = fitg$TMBfn
       out$logL <- fitg$logL
-      out$randomB = randomB
       if (num.lv|num.lv.c > 0)
         out$lvs <- fitg$lvs
       # out$X <- fitg$X
       
-
       out$params <- fitg$params
-      if (sd.errors) {
-        out$sd <- fitg$sd
-        if(!is.null(fitg$sd)&(num.lv.c+num.lv)>0|!is.null(fitg$sd)&row.eff=="random"){
-          if(!is.finite(determinant(fitg$Hess$cov.mat.mod)$modulus)){
+      
+      #### Try to calculate sd errors
+      if (!is.infinite(out$logL) && sd.errors) {
+        trsd <- try({
+          ses <- se.gllvm(out)
+          out$sd <- ses$sd
+          out$Hess <- ses$Hess
+          out$prediction.errors <- ses$prediction.errors
+        if(!is.null(out$sd)&(num.lv.c+num.lv)>0|!is.null(out$sd)&row.eff=="random"){
+          if(!is.finite(determinant(out$Hess$cov.mat.mod)$modulus)){
             warning("Determinant of the variance-covariance matix is zero. Please double check your model for e.g. overfitting or lack of convergence. \n")
           }
         }
-        
+        }, silent = TRUE)
+        if(inherits(trsd, "try-error")) { cat("Standard errors for parameters could not be calculated, due to singular fit.\n") }
       }
+      
       if (family == "tweedie") {
         out$Power <- fitg$Power
       }
+      
       if(family == "ordinal"){
         out$zeta.struc = zeta.struc
       }
@@ -1316,7 +1390,8 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
       if(num.lv.c>0|num.RR>0){
         cat("Try scaling and centering your predictors before entering them into the model, if you haven't. \n")
       }
-      }
+    }
+    if(family == "binomial")out$Ntrials = fitg$Ntrials
     if (is.null(out$terms) && return.terms)
       out$terms <- fitg$terms
     if (is.finite(out$logL) && !is.null(TR) && NCOL(out$TR)>0 && NCOL(out$X)>0) {
@@ -1356,7 +1431,7 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     # check if quadratic coefs have converged or have stuck to "LV"
     if(isTRUE(quadratic)){
       if(length(unique(round(out$params$theta[,-c(1:(num.RR+num.lv.c+num.lv)),drop=F],6)))==(num.RR+num.lv.c+num.lv)){
-        warning("Quadratic model seems to have converged to species-common tolerances. Try refitting with different starting values or to change the optimizer.\n")
+        warning("Quadratic model seems to have converged to species-common tolerances. Try refitting with `start.struc='all'`, or with different starting values.\n")
       out$quadratic <- "LV"        
       }else if(length(unique(out$params$theta[,-c(1:(num.RR+num.lv.c+num.lv)),drop=F]))==1 && starting.val == "zero"){
         warning("It looks like the optimizer failed to move the quadratic coefficients away from the starting values. Please change the starting values. \n")
@@ -1375,9 +1450,10 @@ gllvm <- function(y = NULL, X = NULL, TR = NULL, data = NULL, formula = NULL, fa
     }
     if(!TMB&family=="ordinal"){
       out$zeta.struc <- "species"
+    }else if(TMB & family == "ordinal"){
+      out$zeta.struc = fitg$zeta.struc
     }
-    out$Hess = fitg$Hess
-    out$prediction.errors = fitg$prediction.errors
+
     out$call <- match.call()
     if(quadratic == FALSE){
       class(out) <- "gllvm"  
